@@ -1,7 +1,8 @@
 """Utils for Matter server (and client)."""
 from __future__ import annotations
 
-from base64 import b64decode, b64encode
+from base64 import b64decode
+import binascii
 from dataclasses import MISSING, asdict, fields, is_dataclass
 from datetime import datetime
 from enum import Enum
@@ -10,17 +11,38 @@ from importlib.metadata import PackageNotFoundError, version as pkg_version
 import logging
 import platform
 from types import NoneType, UnionType
-from typing import Any, TypeVar, Union, get_args, get_origin, get_type_hints
+from typing import (
+    TYPE_CHECKING,
+    Any,
+    TypeVar,
+    Union,
+    get_args,
+    get_origin,
+    get_type_hints,
+)
 
-from chip.clusters.Types import Nullable, NullValue
+from chip.clusters.ClusterObjects import ClusterAttributeDescriptor
+from chip.clusters.Types import Nullable
 from chip.tlv import float32, uint
 
-_T = TypeVar("_T")
+if TYPE_CHECKING:
+    from _typeshed import DataclassInstance
+
+    _T = TypeVar("_T", bound=DataclassInstance)
 
 CHIP_CLUSTERS_PKG_NAME = "home-assistant-chip-clusters"
 CHIP_CORE_PKG_NAME = "home-assistant-chip-core"
 
 
+def create_attribute_path_from_attribute(
+    endpoint_id: int, attribute: ClusterAttributeDescriptor
+) -> str:
+    """Create path/identifier for an Attribute."""
+    return create_attribute_path(
+        endpoint_id, attribute.cluster_id, attribute.attribute_id
+    )
+
+
 def create_attribute_path(endpoint: int, cluster_id: int, attribute_id: int) -> str:
     """
     Create path/identifier for an Attribute.
@@ -37,44 +59,17 @@ def parse_attribute_path(attribute_path: str) -> tuple[int, int, int]:
     return (int(endpoint_id_str), int(cluster_id_str), int(attribute_id_str))
 
 
-def dataclass_to_dict(obj_in: object, skip_none: bool = False) -> dict:
-    """Convert dataclass instance to dict, optionally skip None values."""
-    if skip_none:
-        dict_obj = asdict(
-            obj_in, dict_factory=lambda x: {k: v for (k, v) in x if v is not None}
-        )
-    else:
-        dict_obj = asdict(obj_in)
+def dataclass_to_dict(obj_in: DataclassInstance) -> dict:
+    """Convert dataclass instance to dict."""
 
-    def _convert_value(value: Any) -> Any:
-        """Do some common conversions."""
-        if isinstance(value, list):
-            return [_convert_value(x) for x in value]
-        if isinstance(value, Nullable) or value == NullValue:
-            return None
-        if isinstance(value, dict):
-            return _clean_dict(value)
-        if isinstance(value, Enum):
-            return value.value
-        if isinstance(value, bytes):
-            return b64encode(value).decode()
-        if isinstance(value, float32):
-            return float(value)
-        if type(value) == type:
-            return f"{value.__module__}.{value.__qualname__}"
-        if isinstance(value, Exception):
-            return None
-        return value
-
-    def _clean_dict(_dict_obj: dict) -> dict:
-        _final = {}
-        for key, value in _dict_obj.items():
-            if isinstance(key, int):
-                key = str(key)
-            _final[key] = _convert_value(value)
-        return _final
-
-    return _clean_dict(dict_obj)
+    return asdict(
+        obj_in,
+        dict_factory=lambda x: {
+            # ensure the dict key is a string
+            str(k): v
+            for (k, v) in x
+        },
+    )
 
 
 def parse_utc_timestamp(datetime_string: str) -> datetime:
@@ -94,11 +89,8 @@ def parse_value(name: str, value: Any, value_type: Any, default: Any = MISSING)
         if hasattr(value_type, "from_dict"):
             return value_type.from_dict(value)
         # handle a parse error in the sdk which is returned as:
-        # {'TLVValue': None, 'Reason': None}
-        if (
-            value.get("TLVValue", MISSING) is None
-            and value.get("Reason", MISSING) is None
-        ):
+        # {'TLVValue': None, 'Reason': None} or {'TLVValue': None}
+        if value.get("TLVValue", MISSING) is None:
             if value_type in (None, Nullable, Any):
                 return None
             value = None
@@ -111,13 +103,14 @@ def parse_value(name: str, value: Any, value_type: Any, default: Any = MISSING)
         return None
     if is_dataclass(value_type) and isinstance(value, dict):
         return dataclass_from_dict(value_type, value)
-    origin = get_origin(value_type)
-    if origin is list and isinstance(value, list):
-        return [
+    # get origin value type and inspect one-by-one
+    origin: Any = get_origin(value_type)
+    if origin in (list, tuple) and isinstance(value, list | tuple):
+        return origin(
             parse_value(name, subvalue, get_args(value_type)[0])
             for subvalue in value
             if subvalue is not None
-        ]
+        )
     # handle dictionary where we should inspect all values
     elif origin is dict:
         subkey_type = get_args(value_type)[0]
@@ -154,13 +147,19 @@ def parse_value(name: str, value: Any, value_type: Any, default: Any = MISSING)
         return None
     elif origin is type:
         return get_type_hints(value, globals(), locals())
+    # handle Any as value type (which is basically unprocessable)
     if value_type is Any:
         return value
+    # raise if value is None and the value is required according to annotations
     if value is None and value_type is not NoneType:
         raise KeyError(f"`{name}` of type `{value_type}` is required.")
 
     try:
         if issubclass(value_type, Enum):
+            # handle enums from the SDK that have a value that does not exist in the enum (sigh)
+            if value not in value_type._value2member_map_:
+                # we do not want to crash so we return the raw value
+                return value
             return value_type(value)
         if issubclass(value_type, datetime):
             return parse_utc_timestamp(value)
@@ -173,8 +172,14 @@ def parse_value(name: str, value: Any, value_type: Any, default: Any = MISSING)
         return float(value)
     if value_type is int and isinstance(value, str) and value.isnumeric():
         return int(value)
+    # handle bytes values (sent over the wire as base64 encoded strings)
     if value_type is bytes and isinstance(value, str):
-        return b64decode(value.encode())
+        try:
+            return b64decode(value.encode("utf-8"))
+        except binascii.Error:
+            # unfortunately sometimes the data is malformed
+            # as it is not super important we ignore it (for now)
+            return b""
 
     # Matter SDK specific types
     if value_type is uint and (
